/*
    ettercap -- GeoIP interface; IPv4/6 address to geolocation lookup.

    Copyright (C) ALoR & NaGA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

#include <ec.h>
#include <ec_geoip.h>

#ifdef WITH_GEOIP

#include <assert.h>
#include <GeoIP.h>

/* libGeoIP private functions.
 */
extern void _GeoIP_setup_dbfilename (void);
extern int  GeoIP_is_private_ipnum_v4 (unsigned long ipnum);

static GeoIP *geoip_country_ctx  = NULL;
static char  *geoip_country_info = NULL;
static int    geoip_country_db   = 0;

static void ec_geoip_exit (void)
{
  SAFE_FREE (geoip_country_info);
  GeoIP_delete (geoip_country_ctx);
  GeoIP_cleanup();
  geoip_country_info = NULL;
  geoip_country_ctx = NULL;
}

void ec_geoip_init (void)
{
  // try to find and open it in the default location
  geoip_country_ctx = GeoIP_new(GEOIP_COUNTRY_EDITION);
  // not found, fallback in the configuration file value
  if(!geoip_country_ctx) {
    char file [PATH_MAX];

    if (!GBL_CONF->geoip.data_file)
       return;

    strlcpy (file, GBL_CONF->geoip.data_file, sizeof(file));
    _GeoIP_setup_dbfilename();

    geoip_country_ctx = GeoIP_open (file, GEOIP_COUNTRY_EDITION);
    if (!geoip_country_ctx)
    {
      DEBUG_MSG ("%s not found.\n", file);
      GeoIP_cleanup();
      return;
    }
  }
  geoip_country_db   = GeoIP_database_edition (geoip_country_ctx);
  geoip_country_info = GeoIP_database_info (geoip_country_ctx);

  DEBUG_MSG ("Description: %s.", GeoIPDBDescription[geoip_country_db]);
  DEBUG_MSG ("Info:        %s. Countries: %u\n",
             geoip_country_info ? geoip_country_info : "<none>", GeoIP_num_countries());

  atexit (ec_geoip_exit);
}

int ec_get_country_from_ip (const struct ip_addr *ip,
                            const char **country_code,
                            const char **country_name)
{
  unsigned long ipnum;
  int id;

  /* To-do: handle IPv6 addresses too.
   */
  assert (ntohs(ip->addr_type) == AF_INET);

  if (ip_addr_is_zero((struct ip_addr*)ip)) {
    *country_code = "00";
    return (E_SUCCESS);
  }

  ipnum = ntohl (*(unsigned long*)&ip->addr);
  if (GeoIP_is_private_ipnum_v4(ipnum)) {
    if (country_code)
       *country_code = "--";
    if (country_name)
       *country_name = "--";
    return (E_SUCCESS);
  }

  /* If these fails, the above init failed.
   */
  if (geoip_country_db != GEOIP_COUNTRY_EDITION || !geoip_country_ctx)
     return (E_INITFAIL);

  id = GeoIP_id_by_ipnum (geoip_country_ctx, ipnum);
  if (country_code)
     *country_code = GeoIP_code_by_id (id);
  if (country_name)
     *country_name = GeoIP_name_by_id (id);
  return (E_SUCCESS);
}

#else   /* !WITH_GEOIP */

void ec_geoip_init (void)
{
}

int ec_get_country_from_ip (const struct ip_addr *ip,
                            const char **country_code,
                            const char **country_name)
{
  (void) ip;

  if (country_code)
     *country_code = "--";
  if (country_name)
     *country_name = "--";
  return (E_SUCCESS);
}
#endif  /* WITH_GEOIP */

